####################
# zope_test        #
# (c) 2002 Naumen  #
####################
"""
Revision information:
$Id: ZopeTest.py,v 1.2 2002/08/26 05:10:51 ikuleshov Exp $

Zope unit-testing framework
"""

__version__ = '$Revision: 1.2 $'[11:-2]


from sys import path, stderr, stdin
from os import environ, chdir, getcwd, listdir
from os.path import walk
from unittest import TestCase, TextTestRunner, TestSuite, makeSuite

UT_makeSuite = makeSuite


class ZopeTestRunner(TextTestRunner):
    """
    Runner for Zope unit-tests.
    
    ZopeTestRunner passes the application (Zope hierarchy root object) to all
    ZopeTestCases it runs which makes Zope testing possible.
    """
    def __init__(self, stream=stderr, descriptions=1, verbosity=1, zopeLibPath=None):
        TextTestRunner.__init__(self, stream, descriptions, verbosity)

        if zopeLibPath is None:
            zopeRoot = environ.get('ZT_ZOPE_ROOT', None)
            if zopeRoot is not None:
                zopeLibPath = zopeRoot + '/lib/python'
            else:
                zopeLibPath = environ.get('HOME', '') + '/zope/lib/python'
        path.append(zopeLibPath)

        import ZODB
        import Zope

        self.app = Zope.app()

        stderr.write('Zope env started\n')

    def __del__(self):
        self.app._p_jar.close()

    def _makeResult(self):
        result = TextTestRunner._makeResult(self)
        result.app = self.app
        result.stream = self.stream
        return result


class ZopeTestCase(TestCase):
    """
    base class for all Zope test cases.
    
    'app' attribute is application (Zope hierarchy root object), which comes
    handy in Zope unit-tests.
    """
    def __call__(self, result=None):
        from Testing.makerequest import makerequest
        if result is not None:
            if hasattr(result, 'app'):
                self.app = makerequest(result.app, result.stream)
            return TestCase.__call__(self, result)
        else:
            return TestCase.__call__(self, result)


def makeSuite(tests):
    """
    makes a suite out of test class, list of classes, or list of suites
    """
    if type(tests) == type([]):
        return TestSuite(map(lambda i: makeSuite(i), tests))
    elif isinstance(tests, TestSuite):
        return tests
    else:
        test_prefix = getattr(tests, 'test_prefix', 'test')
        return UT_makeSuite(tests, test_prefix)


def runTests(tests, stream=stderr, descriptions=1, verbosity=1, zopeLibPath=None):
    """
    Runs tests in Zope environment
    
    'tests' can be a class, suite, or a list of classes or suites.
    If zopeLibPath is unspecified, checks ZT_ZOPE_ROOT env var and if found,
    assumes that it contains a path to the Zope root. Otherwise tries to
    find Zope at $HOME/zope
    """
    ztr = ZopeTestRunner(stream, descriptions, verbosity, zopeLibPath)
    return ztr.run(makeSuite(tests))


def loadTestsFromModule(module):
    """
    load tests from module

    scans all module for ZopeTestCase-derived classes and returns a suite made
    out of the tests in those classes
    """
    mod = __import__(module)
    testCases = []
    for i in mod.__dict__.values():
        try:
            if issubclass(i, ZopeTestCase):
                testCases.append(i)
        except TypeError:
            pass
    
    return makeSuite(testCases)


def loadTestsFromDir(dir='.'):
    """
    load tests from directory

    scans all .py files in directory for ZopeTestCase-derived classes and
    returns a suite made out of the tests in those classes
    """
    oldDir = getcwd()
    chdir(dir)
    tests = []
    for mod in filter(lambda i: i.endswith('Tests.py'), listdir('.')):
        tests.append(loadTestsFromModule(mod[:-3]))
    chdir(oldDir)
    return makeSuite(tests)


def loadTestsFromTree(root='.'):
    """
    load tests from a directory tree

    scans all .py files in 'tests' directories in directory tree for
    ZopeTestCase-derived classes and returns a suite made out of the tests in
    those classes
    """
    tests = []
    def collectTests(tests, dirname, names):
        if dirname.endswith('/tests'):
            tests.append(loadTestsFromDir(dirname))

    walk(root, collectTests, tests)
    return makeSuite(tests)
